'''
Created on Jun 2, 2016

@author: scrosby
'''
from gus.Gus import Client

class ImportClient(Client):
    def create_user_story(self, product_tag_id, subject, status, **kwargs):
        new_work = kwargs
        new_work['Product_Tag__c'] = product_tag_id
        new_work['Subject__c'] = subject
        new_work['Status__c'] = status
        
        result = self.sf_session.ADM_Work__c.create(new_work)
        self.sf_session.ADM_Work__c.update(result['id'], {'Type__c':'User Story', 'Created_On_import__c':new_work['Created_On_import__c']})
        return result['id']
    
    def create_bug(self, product_tag_id, subject, status, found_in_build, impact, frequency, **kwargs):
        new_work = kwargs
        new_work['Product_Tag__c'] = product_tag_id
        new_work['Subject__c'] = subject
        new_work['Status__c'] = status
        new_work['Found_in_Build__c'] = found_in_build
        new_work['Impact__c'] = impact
        new_work['Frequency__c'] = frequency
        
        result = self.sf_session.ADM_Work__c.create(new_work)
        self.sf_session.ADM_Work__c.update(result['id'], {'Created_On_import__c':new_work['Created_On_import__c']})
        return result['id']
    
    def create_investigation(self, product_tag_id, subject, status, **kwargs):
        new_work = kwargs
        new_work['Product_Tag__c'] = product_tag_id
        new_work['Subject__c'] = subject
        new_work['Status__c'] = status
        new_work['Type__c'] = 'Investigation'
        
        result = self.sf_session.ADM_Work__c.create(new_work)
        self.sf_session.ADM_Work__c.update(result['id'], {'Type__c':'Investigation','Created_On_import__c':new_work['Created_On_import__c']})
        return result['id']
    
    def create_build(self, name):
        result = self.sf_session.ADM_Build__c.create({'name':name})
        return result['id']
    
    def create_task(self, work_id, content, status, assignment, comment=None, hours=0):
        remaining = hours
        actual = hours
        if status == 'Completed':
            remaining = 0
        result = self.sf_session.ADM_Task__c.create({'Work__c': work_id, 'Subject__c': content, 'Status__c': status, 'Assigned_To__c': assignment, 'Comments__c': comment, 'Starting_Hours__c': hours, 'Hours_Remaining__c': remaining, 'Actual_Hours__c': actual})
        return result['id']
    
    def create_acceptance_criteria(self, work_id, summary, detail):
        result = self.sf_session.ADM_Acceptance_Criterion__c.create({'Work__c': work_id, 'Name': summary, 'Description__c': detail})
        return result['id']
    
    def create_epic(self, name, description, team, owner=None):
        if owner:
            result = self.sf_session.ADM_Epic__c.create({'Name': name, 'Description__c': description, 'Team__c': team, 'OwnerId': owner})
        else:
            result = self.sf_session.ADM_Epic__c.create({'Name': name, 'Description__c': description, 'Team__c': team})
            
        return result['id']
    
    def update_epic_owner(self, epic_id, owner_id):
        self.sf_session.ADM_Epic__c.update(epic_id, {'OwnerId': owner_id})
        
    def update_create_date(self, work_id, create_date):
        self.sf_session.ADM_Work__c.update(work_id, {'Created_On_import__c': create_date})
    
    def link_work_to_epic(self, work_id, epic_id):
        self.sf_session.ADM_Work__c.update(work_id, {"Epic__c": epic_id})
        
    def get_existing_keys(self, product_tag, key_prefix):
        result = self.sf_session.query("Select FTest__c from ADM_Work__c where Product_Tag__c='{}' and FTest__c like '{}-%' and Status__c not in('Never','Rejected')".format(product_tag, key_prefix))
        return [x['ftest__c'] for x in result['records']]
    
    def find_work_by_jira_id(self, key):
        result = self.sf_session.query("Select Id from ADM_Work__c where FTest__c = '{}'".format(key))
        if len(result['records']) >= 1:
            return result['records'][0]['Id']
        else:
            return None
        
    def update_description(self, work_id, desc):
        self.sf_session.ADM_Work__c.update(work_id, {'Details_and_Steps_to_Reproduce__c':desc})
    
    def add_child_work(self, parent, child):
        relationship = {
                'Parent_Work__c': parent,
                'Child_Work__c': child
                }
        result = self.sf_session.ADM_Parent_Work__c.create(relationship)
        return result['id']
    
    def get_assignment_rule_id(self, product_tag, rule_type='Investigation'):
        result = self.sf_session.query("Select Assignment_Rule__c from ADM_Tag_Assignment__c where Product_Tag__c = '{}' and Record_Type__c = '{}'".format(product_tag, rule_type))
        if len(result['records']) == 1:
            return result['records'][0]['Assignment_Rule__c']
        else:
            return None
        
    def get_assignment_rule_record(self, rule_id):
        return self.sf_session.ADM_Assignment_Rule__c.get(rule_id)
    
    def get_investigation_assignment_rule(self, product_tag):
        record_id = self.get_assignment_rule_id(product_tag)
        if record_id:
            return self.get_assignment_rule_record(record_id)
        else:
            return None
        
    def get_bug_assignment_rule(self, product_tag):
        record_id = self.get_assignment_rule_id(product_tag, rule_type="Bug")
        if record_id:
            return self.get_assignment_rule_record(record_id)
        else:
            return None

    def get_story_assignment_rule(self, product_tag):
        record_id = self.get_assignment_rule_id(product_tag, rule_type="User Story")
        if record_id:
            return self.get_assignment_rule_record(record_id)
        else:
            return None
        